﻿# Copyright © Biomni Ltd


$VerbosePreference = "SilentlyContinue"
Import-Module "$PSScriptRoot\..\FrontOffice.psm1"
Import-Module "$PSScriptRoot\..\MachineDB.psm1"
Import-Module "$PSScriptRoot\..\FlexiField.psm1"
Import-Module "$PSScriptRoot\..\..\bin\NetBackupAdapterApi.Sdk.dll"

$VerbosePreference = "Continue"


switch($FrontOffice.FlexiFieldOperation) {
	'GetConfiguration' {
		#Populate Configuration data including column names and filtering
		$FrontOffice.FlexiFieldConfiguration.PagingEnabled = $true
		$FrontOffice.FlexiFieldConfiguration.SearchingEnabled = $true
		$FrontOffice.FlexiFieldConfiguration.UserFilteringEnabled = $true		
		$FrontOffice.FlexiFieldConfiguration.AddColumn('Id', 'Machine Id', 0, 'String')
		$FrontOffice.FlexiFieldConfiguration.AddColumn('Code', 'Machine Code', 0, 'String')
		$FrontOffice.FlexiFieldConfiguration.AddColumn('Name', 'Name', 200, 'String')
        $FrontOffice.FlexiFieldConfiguration.AddColumn('Registered', 'Registered for file restore', 0, 'String')
        $FrontOffice.FlexiFieldConfiguration.AddColumn('ClientName', 'NetBackup Client Name', 0, 'String')
        $FrontOffice.FlexiFieldConfiguration.AddColumn('VMDisplayName', 'VM Display Name', 0, 'String')
        $FrontOffice.FlexiFieldConfiguration.AddColumn('CatalogName', 'NetBackup Catalog Name', 200, 'String')
		$FrontOffice.FlexiFieldConfiguration.KeyColumnName = 'Code'
		$FrontOffice.FlexiFieldConfiguration.DisplayColumnName = 'Name'
	}



	'GetData' {

        #Update the Request with ID and status, clear fields in case it is a copy request
        $request = $dapi.GetRequestV5($FrontOffice.RequestId)
        #Get the source VM from the Front Office request.
        $sourceVMField = $request.Fields.FindFlexiFieldByFieldCode('MACHINE')
        #Get machine code of source VM
        $machineCode = $sourceVMField.Rows.Code
        #Get customer code to filter by tenant ownership
        $customerCode = $request.Client.CustomerCode
        #Get requested for id to filter by user visibility
        $requestedFor = $request.RequestedFor.Id

        $requestedForUser = $dapi.getUser($requestedFor)

        if ($requestedForUser.AccessProfileId -eq 'BIOMNI_CLIENTUSER'){
            #SQL query for the selection of machines
            $sqlQuery = "select * from machine where customercode = '$customerCode' and isVisibleToAllUsers = 'true' and MachineCode <> '$machineCode' union all select mc.[MachineId],mc.[MachineCode],mc.[LastSuccessfulBackupDate],mc.[TrafficLightStatus],mc.[CustomerCode],mc.[CatalogName],mc.[LastUpdatedDateTime],mc.[SyncNetBackupData],mc.[ThresholdExpireDateTime],mc.[ThresholdExpired],mc.[SyncPriorityId],mc.[SyncLockedUntil],mc.[SyncLastError],mc.[SyncLastUpdatedDateTime],mc.[Location],mc.[Hardware],mc.[OS],mc.[NetBackupClientName],mc.[VMDisplayName],mc.[IsVisibleToAllUsers],mc.[DisplayName],mc.[ImportSource],mc.[IsInVCloud],mc.[AdditionalData],mc.[CreatedDateTime],mc.[IsDeletedFromImportSource],mc.[ProtectionTypeId],mc.[ContainerVAppId] from machine mc inner join MachineUser mcu on mc.MachineId = mcu.MachineId where mcu.UserId = '$requestedFor' and customercode = '$customerCode' and isVisibleToAllUsers = 'false' and MachineCode <> '$machineCode'"

        }
        else{
            #SQL query for the selection of machines
            $sqlQuery = "select * from machine where customercode = '$customerCode' and MachineCode <> '$machineCode'"

        }

        #Call function to process query and return objects
        $getMachinesResult = $serviceFactory.AdminMachineService.GetAllQueryableForOData().SqlQuery($sqlQuery)
        
        #Get count of machines
        $totalRecords = $getMachinesResult.count
        #Loop through machines, adding each row to external list box
        if ($totalRecords -gt 0)
		{
		 	$getMachinesResult | sort { $_.MachineCode } | foreach {

				$row = @{}
				$row.Id = $_.MachineId
				$row.Code = $_.MachineCode
				$row.Name = $_.DisplayName
                $row.VMDisplayName = $_.VMDisplayName
                $row.CatalogName = $_.CatalogName
                if ($_.NetBackupClientName )
                {
                    if ($_.IsVisibleToAllUsers -eq $true) {
                        $row.Registered = 'Yes'
                        $row.ClientName = $_.NetBackupClientName
                        $FrontOffice.FlexiFieldData.Add($row)
                    }
                    else {
                            $row.Registered = 'No'
                            $row.ClientName = $_.NetBackupClientName
                            $FrontOffice.FlexiFieldData.Add($row)
                    }

                } 
                else 
                {
                    if ($_.IsVisibleToAllUsers -eq $true) {
                        $row.ClientName =''
                        $row.Registered = 'No2'
                        $FrontOffice.FlexiFieldData.Add($row)
                    }
                    else {
                        $row.ClientName =''
                        $row.Registered = 'No3'
                        $FrontOffice.FlexiFieldData.Add($row)

                    }

                }
         
                
		      }

            }

		}


	'GetFilters' { 

	}
}

