﻿# Copyright © Biomni Ltd

## Set Script Root when in ISE
$scriptRoot = $PSScriptRoot
if (-not $scriptRoot) {
    $scriptRoot = '.'
}


$VerbosePreference = "SilentlyContinue"
Import-Module "$PSScriptRoot\..\..\bin\NetBackupAdapterBusinessObjects.dll"
Import-Module "$PSScriptRoot\..\..\bin\NetBackupAdapterServices.dll"
Import-Module "$PSScriptRoot\..\FrontOffice.psm1"
Import-Module "$PSScriptRoot\..\MachineDB.psm1"
Import-Module "$PSScriptRoot\..\NetBackupApi.psm1"
$VerbosePreference = "Continue"

<#
############################################
## Start Setup of Dev Environment
## Only used when Devloping in ISE
## When in production this should all be commented out
## as it will be setup by Initial.ps1

## Change these connection strings to point to your NSS Database and Front Office DAPI
[NetBackupAdapterBusinessObjects.ApplicationConfig]::AdapterDBConnectionString = 'Server=BCC;Database=NetBackupSelfServiceNetbackupAdapter;User ID=NetBackupSelfServiceNetbackupAdapterUser;Password=;Trusted_Connection=false'
[NetBackupAdapterBusinessObjects.ApplicationConfig]::DapiConnectionString = 'authtype=Anonymous;url=http://BCC.biomni.com/NetBackupSelfServicePublicWebService/DirectaApi.svc;'

[void][NetBackupAdapterBusinessObjects.Ioc.UnityConfig]::DisposeContainer()
[void][NetBackupAdapterBusinessObjects.Ioc.UnityConfig]::SetConfiguredContainerForTaskEngine()

## The following variables are initialized in the Initial.ps1 script of each Adapter type
$serviceFactory = [NetBackupAdapterBusinessObjects.Services.ServiceFactory]::Create()
$dapi = $serviceFactory.DirectaApi
$FrontOffice = @{}
$FrontOffice.RequestId = 1833

## End Setup of Dev Environment
############################################
#>

function Is-WindowsPath {
	param(
		[string]$path
	)

	return $path -cmatch "^/[A-Z]/"
}

function Is-WindowsPathFormat {
	param(
		[string]$path
	)

	return $path -cmatch "[A-Z]:\\"
}

function ConvertTo-WindowsPath {
	param(
		[string]$UnixPath
	)

	if (Is-WindowsPath $UnixPath) {
		$folderFile = ""
		$driveLetter = $UnixPath.Substring(1,1)
		if($UnixPath.Length -gt 3) {
			$folderFile = $UnixPath.Substring(3).Replace('/','\')
		}

		return $driveLetter + ":\" + $folderFile
	}
	else {
		return $UnixPath
	}
}

function ConvertTo-UnixPath {
	param(
		[string]$WindowsPath
	)

		$folderFile = ""
		$driveLetter = $WindowsPath.Substring(0,1)
		if($WindowsPath.Length -gt 3) {
			$folderFile = $WindowsPath.Substring(3).Replace('\','/')
		

		return '/' + $driveLetter + "/" + $folderFile
	
	}
}

function GetPolicyTypeCode {
	param(
		[string]$policyType
	)

    switch ($policyType) 
        { 
            "MS-Windows" {return 13} 
            "Standard" {return 0} 
            "VMWare" {return 40} 
            "Hyper-V" {return 41} 
            default {return 40}
        }
}

$fileRestore = [NetBackupAdapterServices.Code.FileRestore]

#Update the Request with ID and status, clear fields in case it is a copy request
Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{'Status'='' }
$request = $dapi.GetRequestV5($FrontOffice.RequestId)
#Get the source VM from the Front Office request.
$sourceVMField = $request.Fields.FindFlexiFieldByFieldCode('MACHINE')
$machineCode = $sourceVMField.Rows.Code

$adapterSettings = $serviceFactory.FrontOfficeService.GetUserAdapterSettings($request.RequestedFor.Id)
$serviceFactory.FrontOfficeService.GetUserAdapterSettings($request.RequestedFor.Id).UserAccessRights.AssertCanRestoreFiles()

$customerCode = $request.Client.CustomerCode
$machine = Get-TenantUserMachine -MachineCode $machineCode -CustomerCode $customerCode -User $request.RequestedFor -ServiceFactory $serviceFactory

$selectedBackup =  $request.Fields.FindFlexiFieldByFieldCode('RESTDATE').Rows

$policyTypeCode = GetPolicyTypeCode($selectedBackup.PolicyType)



$serviceFactory.FrontOfficeService.AssertIsLocationValid($selectedBackup.Location)



[bool]$restoreToOriginalLocation = $request.Fields.FindByFieldCode('ORIG_LOC').Value -eq 'Yes'
if($restoreToOriginalLocation) {
	$restoreSubFolderName = $null
    $suggestedPathFormat = $null
} else {
	$restoreSubFolderName = $fileRestore::GetRestoreSubFolderName($FrontOffice.RequestId)
    $suggestedPathFormat = ($request.Fields.FindByFieldCode('RestDisp').Value -split '\n')[0]
}

[bool]$restoreToAltPath = $request.Fields.FindByFieldCode('ALTPATHQ').Value -eq 'Yes'
[bool]$restoreToAltHost = $request.Fields.FindByFieldCode('ALTHOSTQ').Value -eq 'Yes'
$altHost = $request.Fields.FindByFieldCode('ALTHOST').Value

$fileList = New-Object System.Collections.Generic.List[NetBackupApi.RestoreFileDetail]
$selectionMode = $request.Fields.FindByFieldCode('SelectMode').Value

switch ($selectionMode) {
	'Browse' {
		$treeNodes = $request.Fields.FindTreeViewFieldByFieldCode('BrowseFile')
		foreach($selectedNode in $treeNodes.GetSelectedNodes()) {
            $altPath = $request.Fields.FindByFieldCode('ALTPATH').Value

            if ($altPath.EndsWith('/') -or $altPath.EndsWith('\')){
                $altPath = $altPath.substring(0, $altPath.length-1)
            }

			$restoreFileDetail = New-Object "NetBackupApi.RestoreFileDetail"
			$restoreFileDetail.Source = $selectedNode.GetAttribute('FullPath')

			if($restoreToOriginalLocation) {
				$restoreFileDetail.Destination = $null
			}else{
                if ($restoreToAltPath){

	                if (Is-WindowsPathFormat($suggestedPathFormat)) {

                        $altPath = ConvertTo-UnixPath($altPath)
                        $restoreFileDetail.Destination = $altPath + $restoreFileDetail.Source.substring(2)
		                
	                }
                    else{

                        $restoreFileDetail.Destination = "/" + $altPath + $restoreFileDetail.Source.substring(0)

                    }
                }
                else{
       				$restoreFileDetail.Destination = $fileRestore::GetNewRestoreLocation($restoreFileDetail.Source, $restoreSubFolderName)
                }
			}
			$fileList.Add($restoreFileDetail)
		}
	}
	'Search' {
		$searchField = $request.Fields.FindFlexiFieldByFieldCode('SearchFile')
		foreach($row in $searchField.Rows) {
            $altPath = $request.Fields.FindByFieldCode('ALTPATH').Value

            if ($altPath.EndsWith('/') -or $altPath.EndsWith('\')){
                $altPath = $altPath.substring(0, $altPath.length-1)
            }

			$restoreFileDetail = New-Object "NetBackupApi.RestoreFileDetail"
			$restoreFileDetail.Source = $row.FileOrFolder
			if($restoreToOriginalLocation) {
				$restoreFileDetail.Destination = $null
			}else{
                if ($restoreToAltPath){
	                if (Is-WindowsPathFormat($suggestedPathFormat)) {

                        $altPath = ConvertTo-UnixPath($altPath)
                        $restoreFileDetail.Destination = $altPath + $restoreFileDetail.Source.substring(2)
		                
	                }
                    else{

                        $restoreFileDetail.Destination = "/" + $altPath + $restoreFileDetail.Source.substring(0)

                    }

    				
                }
                else{
       				$restoreFileDetail.Destination = $fileRestore::GetNewRestoreLocation($restoreFileDetail.Source, $restoreSubFolderName)
                }
			}
			$fileList.Add($restoreFileDetail)
		}
	}
}

$masterServerLocation = $adapterSettings.GetMasterServerLocation($machine.Location)
$masterServer = $masterServerLocation.CreateMasterServer()

$sourceVMField = $request.Fields.FindFlexiFieldByFieldCode('ALTHOSTSEL')
$altHost = $sourceVMField.Rows.Code


if ($restoreToAltHost){
    

    $trackingJob = $masterServer.Catalogue.RestoreFiles($filelist, $machine.CatalogName, $altHost, $selectedBackup.BackupTime, [NetBackupApi.PolicyType]::GetById($selectedBackup.PolicyType))

    $activity = $serviceFactory.ActivityService.GetById([int]$request.Fields.ACTIVITYID.Value)
    $activity.TaskData.MachineImageId = $selectedBackup.MachineImageId

    $activity.Status = [NetBackupAdapterBusinessObjects.EntityFramework.Entities.ActivityStatus]::Running
    
    $activity.IsMonitoringRequired = $true
    $activity.UpdateTrackingJob($trackingJob)
    $serviceFactory.ActivityService.Save($activity)

    $serviceFactory.ActivityService.SupersedeFailedActivities($activity, $activity.TaskData)

}
else{

    $activity = $serviceFactory.ActivityService.GetById([int]$request.Fields.ACTIVITYID.Value)
    $activity.TaskData.MachineImageId = $selectedBackup.MachineImageId
    $activity.TaskData.FileDetails = $fileList

    $activity.Status = [NetBackupAdapterBusinessObjects.EntityFramework.Entities.ActivityStatus]::Running
    [void]$serviceFactory.ActivityService.Save($activity)

    $serviceFactory.ActivityService.SupersedeFailedActivities($activity, $activity.TaskData)
    $serviceFactory.TaskEngineService.EnqueueRestoreFile($activity)

}


