﻿# Copyright © Biomni Ltd

$VerbosePreference = "SilentlyContinue"
Import-Module "$PSScriptRoot\..\FrontOffice.psm1"
$VerbosePreference = "Continue"

function Is-WindowsPath {
	param(
		[string]$path
	)

	return $path -cmatch "^/[A-Z]/"
}

function Is-WindowsPathFormat {
	param(
		[string]$path
	)

	return $path -cmatch "[A-Z]:\\"
}

function ConvertTo-WindowsPath {
	param(
		[string]$UnixPath
	)

	if (Is-WindowsPath $UnixPath) {
		$folderFile = ""
		$driveLetter = $UnixPath.Substring(1,1)
		if($UnixPath.Length -gt 3) {
			$folderFile = $UnixPath.Substring(3).Replace('/','\')
		}

		return $driveLetter + ":\" + $folderFile
	}
	else {
		return $UnixPath
	}
}

function ConvertTo-UnixPath {
	param(
		[string]$WindowsPath
	)

		$folderFile = ""
		$driveLetter = $WindowsPath.Substring(0,1)
		if($WindowsPath.Length -gt 3) {
			$folderFile = $WindowsPath.Substring(3).Replace('\','/')
		

		return '/' + $driveLetter + "/" + $folderFile
	
	}
}


$fileRestore = [NetBackupAdapterServices.Code.FileRestore]
$changedFieldCode = $FrontOffice.UiFieldChangedSettings.FieldCode

if ($changedFieldCode -eq 'ORIG_LOC') {

    Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{ 'ALTPATHQ'='No'; 'ALTHOSTQ'='No'}

}


if ($changedFieldCode -eq 'ALTPATHQ') {


    $request = $dapi.GetRequestV5($FrontOffice.RequestId)
    $yn = $request.Fields.FindByFieldCode($changedFieldCode).value
    if ($yn -eq 'Yes') {
        


        $suggestedPathFormat = ($request.Fields.FindByFieldCode('RestDisp').Value -split '\n')[0]

        if (Is-WindowsPathFormat($suggestedPathFormat)) {

            Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{ 'OSMESSAGE'='This restore requires a Windows-style path - an example has been added below'; 'ALTPATH'='e.g. C:\temp\'}

        }
        else{

            Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{ 'OSMESSAGE'='This restore requires a UNIX-style path - an example has been added below'; 'ALTPATH'='e.g. /tmp/'}

        }

    }

}

if ($changedFieldCode -eq 'MACHINE') {
	Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{ 'SearchFile'=''; 'BrowseFile'=''; 'RestDisp'=''; 'OVERWRITE'='False'; 'RESTDATE'='' }
}

if ($changedFieldCode -eq 'RESTDATE') {
	Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{ 'SearchFile'=''; 'BrowseFile'=''; 'RestDisp'=''; 'OVERWRITE'='False' }
}

if ($changedFieldCode -eq 'SelectMode') {
	Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{ 'RestDisp'=''; 'OVERWRITE'='False'; 'ALTPATHQ'='No'; 'ALTHOSTQ'='No' }
	#Note: No need to clear 'SearchFile' or 'BrowseFile' fields since the rules include/ exclude them as opposed to show/hide so data is automatically binned.
}

if ( @('SearchFile', 'BrowseFile') -contains $changedFieldCode ) {
	$request = $dapi.GetRequestV5($FrontOffice.RequestId)
	$newLocations = @()
	$restoreSubFolderName = $fileRestore::GetRestoreSubFolderName($FrontOffice.RequestId)
	switch ($changedFieldCode) {
		'SearchFile' {
			foreach($row in $request.Fields.FindFlexiFieldByFieldCode($changedFieldCode).Rows) {
				$restoreLocation = $fileRestore::GetNewRestoreLocation($row.FileOrFolder, $restoreSubFolderName)
				$restoreLocationDisplay = $fileRestore::ConvertToDisplayPath($restoreLocation)
				$newLocations += "$restoreLocationDisplay"
			}
		}
		'BrowseFile' {
			foreach($selectedNode in $request.Fields.FindTreeViewFieldByFieldCode($changedFieldCode).GetSelectedNodes()) {
				$restoreLocation = $fileRestore::GetNewRestoreLocation($selectedNode.GetAttribute('FullPath'), $restoreSubFolderName)
				$restoreLocationDisplay = $fileRestore::ConvertToDisplayPath($restoreLocation)
				$newLocations += "$restoreLocationDisplay"
			}
		}
	}

	$restoreLocationDisplay = $newLocations -join [System.Environment]::NewLine
	Set-HeaderFieldValues $dapi $FrontOffice.RequestId @{ 'RestDisp'=$restoreLocationDisplay; 'OVERWRITE'='False' }
}